from tensorflow.keras.applications.mobilenet_v2 import preprocess_input
from tensorflow.keras.preprocessing.image import img_to_array
from tensorflow.keras.models import load_model
#from imutils.video import VideoStream
import numpy as np
import argparse
#import imutils
import time
import cv2
import os
#from pretrained_models import get_model

def detect_and_predict_mask(frame, faceNet, maskNet, args):
	# grab the dimensions of the frame and then construct a blob
	# from it
	(h, w) = frame.shape[:2]
	blob = cv2.dnn.blobFromImage(frame, 1.0, (300, 300),
		(104.0, 177.0, 123.0))

	# pass the blob through the network and obtain the face detections
	faceNet.setInput(blob)
	detections = faceNet.forward()

	# initialize our list of faces, their corresponding locations,
	# and the list of predictions from our face mask network
	faces = []
	locs = []
	preds = []

	# loop over the detections
	for i in range(0, detections.shape[2]):
		# extract the confidence (i.e., probability) associated with
		# the detection
		confidence = detections[0, 0, i, 2]

		# filter out weak detections by ensuring the confidence is
		# greater than the minimum confidence
		if confidence > args["confidence"]:
			# compute the (x, y)-coordinates of the bounding box for
			# the object
			box = detections[0, 0, i, 3:7] * np.array([w, h, w, h])
			(startX, startY, endX, endY) = box.astype("int")

			# ensure the bounding boxes fall within the dimensions of
			# the frame
			(startX, startY) = (max(0, startX), max(0, startY))
			(endX, endY) = (min(w - 1, endX), min(h - 1, endY))

			# extract the face ROI, convert it from BGR to RGB channel
			# ordering, resize it to 224x224, and preprocess it
			face = frame[startY:endY, startX:endX]
			face = cv2.cvtColor(face, cv2.COLOR_BGR2RGB)
			face = cv2.resize(face, (224, 224))
			face = img_to_array(face)
			face = preprocess_input(face)

			# add the face and bounding boxes to their respective
			# lists
			faces.append(face)
			locs.append((startX, startY, endX, endY))

	# only make a predictions if at least one face was detected
	if len(faces) > 0:
		# for faster inference we'll make batch predictions on *all*
		# faces at the same time rather than one-by-one predictions
		# in the above `for` loop
		faces = np.array(faces, dtype="float32")
		preds = maskNet.predict(faces, batch_size=32)

	# return a 2-tuple of the face locations and their corresponding
	# locations
	return (locs, preds)

def get_args():
    # construct the argument parser and parse the arguments
    ap = argparse.ArgumentParser()
    ap.add_argument("-i", "--image", required=True,
	    help="path to input image")
    ap.add_argument("-f", "--face", type=str,
	    default="models",
	    help="path to face detector model directory")
    ap.add_argument("-m", "--model", type=str,
	    default="models/helmet_detector.h5",
	    help="path to trained helmet detector model")
    ap.add_argument("-c", "--confidence", type=float, default=0.5,
	    help="minimum probability to filter weak detections")
    args = vars(ap.parse_args())
    return args

def main():
    #get_model()
    print('Downloading models ...')
    args = get_args()
    # load our serialized face detector model from disk
    print("[INFO] loading face detector model...")
    prototxtPath = os.path.sep.join([args["face"], "deploy.prototxt"])
    print(f'Path : {prototxtPath}')
    weightsPath = os.path.sep.join([args["face"],
        "res10_300x300_ssd_iter_140000.caffemodel"])
    print(f'Weights path : {weightsPath}')
    faceNet = cv2.dnn.readNet(prototxtPath, weightsPath)

    # load the face mask detector model from disk
    print("[INFO] loading face mask detector model...")
    maskNet = load_model(args["model"])

    # load the input image from disk, clone it, and grab the image spatial
    # dimensions
    image = cv2.imread(args["image"])
    orig = image.copy()
    (h, w) = image.shape[:2]

    # construct a blob from the image
    blob = cv2.dnn.blobFromImage(image, 1.0, (300, 300),
        (104.0, 177.0, 123.0))

    # pass the blob through the network and obtain the face detections
    print("[INFO] computing face detections...")
    faceNet.setInput(blob)
    detections = faceNet.forward()

    # loop over the detections
    for i in range(0, detections.shape[2]):
        # extract the confidence (i.e., probability) associated with
        # the detection
        confidence = detections[0, 0, i, 2]

        # filter out weak detections by ensuring the confidence is
        # greater than the minimum confidence
        if confidence > args["confidence"]:
            # compute the (x, y)-coordinates of the bounding box for
            # the object
            box = detections[0, 0, i, 3:7] * np.array([w, h, w, h])
            (startX, startY, endX, endY) = box.astype("int")

            # ensure the bounding boxes fall within the dimensions of
            # the frame
            (startX, startY) = (max(0, startX), max(0, startY))
            (endX, endY) = (min(w - 1, endX), min(h - 1, endY))

            # extract the face ROI, convert it from BGR to RGB channel
            # ordering, resize it to 224x224, and preprocess it
            face = image[startY:endY, startX:endX]
            face = cv2.cvtColor(face, cv2.COLOR_BGR2RGB)
            face = cv2.resize(face, (224, 224))
            face = img_to_array(face)
            face = preprocess_input(face)
            face = np.expand_dims(face, axis=0)

            # pass the face through the model to determine if the face
            # has a mask or not
            (mask, withoutMask) = maskNet.predict(face)[0]

            # determine the class label and color we'll use to draw
            # the bounding box and text
            label = "Helmet" if mask > withoutMask else "No Helmet"
            color = (0, 0, 255) if label == "Helmet" else (0, 255, 0)

            # include the probability in the label
            label = "{}: {:.2f}%".format(label, max(mask, withoutMask) * 100)

            # display the label and bounding box rectangle on the output
            # frame
            if mask > withoutMask:
                cv2.putText(image, label, (startX, startY - 10),
                    cv2.FONT_HERSHEY_SIMPLEX, 0.45, color, 2)
                cv2.rectangle(image, (startX, startY), (endX, endY), color, 2)
            else:
                cv2.putText(image, "No helmet detected", (startX, startY - 10),
                    cv2.FONT_HERSHEY_SIMPLEX, 0.45, color, 1)
    # show the output image
    cv2.imshow("Output", image)
    cv2.waitKey(0)

if __name__ == '__main__':
    main()


from setuptools import setup

def readme():
    with open('dxc_helmet/README.md', 'rt') as file:
        return file.read()

setup(
    name='dxc_helmet',
    version='0.1.1',
    description='Helmet detector',
    long_description=readme(),
    long_description_content_type="text/markdown",
    author='Jonathan Fernandes',
    license='MIT',
    packages=['dxc_helmet'],
    install_requires=[]
)


import tensorflow.keras.utils
from collections import namedtuple

#models = namedtuple('models', ['model_name', 'model_url', 'caffe_model', 'caffe_model_url', 'prototxt_name', 'prototxt_url'])
models = namedtuple('models', ['model_name', 'model_url'])
model_name = 'helmet_detector.h5'
model_url = 'https://github.com/jonfernandes/dxc_helmet/raw/master/download/0.1.0/helmet_detector.h5'
#url = 'https://raw.github.com/jonfernandes/dxc_helmet/raw/master/download/0.1.0/helmet_detector.h5'
caffe_model = 'res10_300x300_ssd_iter_140000.caffemodel'
caffe_model_url = 'https://github.com/jonfernandes/dxc_helmet/blob/master/download/0.1.0/res10_300x300_ssd_iter_140000.caffemodel'
prototxt_name = 'deploy.prototxt'
prototxt_url = 'https://github.com/jonfernandes/dxc_helmet/blob/master/download/0.1.0/deploy.prototxt'
'''
tf_27_Aug = models(model_name=model_name, 
                   model_url=model_url,
                   caffe_model=caffe_model, 
                   caffe_model_url=caffe_model_url,
                   prototxt_name=prototxt_name, 
                   prototxt_url=prototxt_url)
'''
tf_27_Aug = models(model_name=model_name, 
                   model_url=model_url)

def get_model(mod=tf_27_Aug):
    helmet = tensorflow.keras.utils.get_file(
        mod.model_name,
        mod.model_url,
        cache_subdir='models'
    )
    '''
    face_model = tensorflow.keras.utils.get_file(
        mod.caffe_model,
        mod.caffe_model_url,
        cache_subdir='models'
    )
    prototxt = tensorflow.keras.utils.get_file(
        mod.prototxt_name,
        mod.prototxt_url,
        cache_subdir='models'
    )
    '''
